bl_info = {
    "name": "Super Parent",
    "description": "",
    "author": "Przemysław Świda (Calm Dude)",
    "version": (1, 5, 1),
    "blender": (2, 90, 0),
    "location": "3D View > Add Menu",
    "warning": "", # used for warning icon and text in addons panel
    "wiki_url": "",
    "tracker_url": "",
    "category": "Object",
    "shortname": 'S_Parent'
}

import bpy
import bmesh
import bpy_extras

from bpy.props import (StringProperty,
                       BoolProperty,
                       IntProperty,
                       FloatProperty,
                       FloatVectorProperty,
                       EnumProperty,
                       PointerProperty,
                       )
                       
from bpy.types import (Panel,
                       Operator,
                       AddonPreferences,
                       PropertyGroup,
                       )
import mathutils
import math
from math import radians
from mathutils import Vector

from bpy_extras import view3d_utils


import bgl
import blf
import gpu
from gpu_extras.batch import batch_for_shader

from bpy.app.handlers import persistent

import os

from bpy.types import (
    Gizmo,
    GizmoGroup,
)
from .translation.translate import GetTranslationDict



addonName = os.path.basename(os.path.dirname(__file__))

# ------------------------------------------------------------------------
# ------------------------------------------------------------------------
#    UI
# ------------------------------------------------------------------------
# ------------------------------------------------------------------------


def draw_superparent_menu(self, context):
    
    pcoll = preview_collections_SuperParent["main"]
    op = self.layout.operator("ap.add_super_parent", text="Super Parent", icon_value=pcoll['super_parent'].icon_id)
    op.name = "Super Parent"
    op.type = get_addon_prefs().type
    op.color = get_addon_prefs().mesh_color

def draw_cage_object(self, context):
    if  get_addon_prefs().automatic_cage == False:
        layout = self.layout
        row = layout.row()
        obj = bpy.context.object
        row.operator("ap.refresh_super_parent_cage", text="Refit Cage", icon = 'FILE_REFRESH')
        row.enabled = obj.isSuperParentCage
        
        layout.separator()


# ------------------------------------------------------------------------
#    Functions and Properties
# ------------------------------------------------------------------------

def get_addon_prefs():
    import os
    addon_name = __package__
    addon_prefs = bpy.context.preferences.addons[addon_name].preferences
    return (addon_prefs)

#--------------------------------------------------------------------------
#         Preferences
#--------------------------------------------------------------------------

class SuperParentPreferences(AddonPreferences):
    
    bl_idname = __package__
    mesh_color: FloatVectorProperty(  
       name="Default Mesh Color",
       subtype='COLOR',
       default=(0.0, 0.715693, 0.98225),
       min=0.0, max=1.0,
       description="color picker"
       )
    type: bpy.props.EnumProperty(
        name="Default Type",
        items = [
        ('FITTED_CAGE','Cage Mesh','','STICKY_UVS_LOC',0),
        ('PLAIN_AXES','Plain Axes','','EMPTY_AXIS',1),
        ('ARROWS','Arrows','','EMPTY_ARROWS',2),
#        ('SINGLE_ARROW','Single Arrow','','EMPTY_SINGLE_ARROW',3),
#        ('CIRCLE','Circle','','MESH_CIRCLE',4),
        ('CUBE','Cube','','CUBE',3),
        ('SPHERE','Sphere','','SPHERE',4),
#        ('CONE','Cone','','CONE',7),
#        ('IMAGE','Image','','IMAGE',8)
        ],
        default = 'FITTED_CAGE'
        ) 
        
    automatic_cage: bpy.props.BoolProperty(default=False, name='Refit Cage Automatically')
    
    def draw(self, context):
        layout = self.layout
        layout.row().prop(self, "mesh_color")
        layout.row().prop(self, "type")
        layout.row().prop(self, "automatic_cage")



# ------------------------------------------------------------------------
# ------------------------------------------------------------------------
#    OPERATORS
# ------------------------------------------------------------------------
# ------------------------------------------------------------------------


class SP_AddSuperParent(Operator):
    bl_idname = "ap.add_super_parent"
    bl_label = "Super Parent" 
    bl_options = {"UNDO", "REGISTER"}
    bl_description = "Add Parent for selected objects, with keeping hierarchy structure"
    type: bpy.props.EnumProperty(
        name="Type",
        items = [
        ('FITTED_CAGE','Cage Mesh','','STICKY_UVS_LOC',0),
        ('PLAIN_AXES','Plain Axes','','EMPTY_AXIS',1),
        ('ARROWS','Arrows','','EMPTY_ARROWS',2),
#        ('SINGLE_ARROW','Single Arrow','','EMPTY_SINGLE_ARROW',3),
#        ('CIRCLE','Circle','','MESH_CIRCLE',4),
        ('CUBE','Cube','','CUBE',3),
        ('SPHERE','Sphere','','SPHERE',4),
#        ('CONE','Cone','','CONE',7),
#        ('IMAGE','Image','','IMAGE',8)
        ],
        default = 'FITTED_CAGE'
        )
    scale: bpy.props.FloatProperty(name="Size", default = 1.0)
    origin_at_cursor: bpy.props.BoolProperty(name="Origin at Cursor", default = False)
    show_in_front: bpy.props.BoolProperty(name="Show in Front", default = True)
    displayColorGizmo: bpy.props.BoolProperty(name="Show Colored Gizmo", default = True)
    parent_whole_collection: bpy.props.BoolProperty(name="Whole Collection", default = False)
    parent_subcollections: bpy.props.BoolProperty(name="Sub-Collections", default = False)
    
    color: FloatVectorProperty(  
       name="Color",
       subtype='COLOR',
       default=(0.0, 0.715693, 0.98225),
       min=0.0, max=1.0,
       description="color picker"
       )
    name: bpy.props.StringProperty(default='Super Parent', name='Name')
        
    @classmethod
    def poll(cls, context):
        return len(context.selected_objects)>0
    
    def draw(self, context):
        layout = self.layout
        
        row = layout.row()
        row.prop(self,'name')
        
        row = layout.row()
        row.prop(self,'type')
        layout.separator()
        
        if self.type != 'FITTED_CAGE':
            row = layout.row()
            row.prop(self,'scale')
            layout.separator()

        row = layout.row()
        row.prop(self,'displayColorGizmo')
        row = layout.row()
        row.prop(self,'color')
        
        row = layout.row()
        row.prop(self,'show_in_front')
        row.enabled = 1- self.displayColorGizmo
        
        row = layout.row()
        row.prop(self,'origin_at_cursor')
        

        
        
        row = layout.row()
        row.prop(self,'parent_whole_collection')
        if self.parent_whole_collection:
            row.prop(self,'parent_subcollections')
    
    def execute(self, context):
                        
        missingObjects = []
        boundBoxXs = []
        boundBoxYs = []
        boundBoxZs = []
        collections = []
        
    
                       
        #find all parents and orphans                 
        for o in context.selected_objects:
            if o.parent == None:
                missingObjects.append(o)
                for uc in o.users_collection:
                    collections.append(uc)
            
        
        if self.parent_whole_collection:
           for c in collections:
                if self.parent_subcollections: #subcollections too
                    for o in c.all_objects:
                        try:
                            o.select_set(True)
                        except:
                            pass
                        if o not in missingObjects and o.parent == None:
                            missingObjects.append(o)
                else:
                    for o in c.objects:
                        o.select_set(True)
                        if o not in missingObjects and o.parent == None:
                            missingObjects.append(o)
                    
                        
        for o in context.selected_objects:
            bb_vertices = [Vector(v) for v in o.bound_box]
            mat = o.matrix_world
            world_bb_vertices = [mat @ v for v in bb_vertices]
            for i in world_bb_vertices:
                boundBoxXs.append(i[0])    
                boundBoxYs.append(i[1])    
                boundBoxZs.append(i[2])
            o.select_set(False)
            
        
        if len(missingObjects)>0:
                
            favCollection = max(set(collections), key = collections.count) 
            
            SPBoxXScale = max(boundBoxXs)- min(boundBoxXs)
            SPBoxYScale = max(boundBoxYs)- min(boundBoxYs)
            SPBoxZScale = max(boundBoxZs)- min(boundBoxZs)
            
            SPBoxScale = (SPBoxXScale/2,SPBoxYScale/2,SPBoxZScale/2)
            SPBoxLocation = (min(boundBoxXs)+SPBoxXScale/2,min(boundBoxYs)+SPBoxYScale/2,min(boundBoxZs)+SPBoxZScale/2)
            
            if self.type in {'PLAIN_AXES', 'ARROWS', 'SINGLE_ARROW', 'CIRCLE', 'CUBE', 'SPHERE', 'CONE', 'IMAGE'}:
                
                obj = bpy.data.objects.new( "empty", None )
                obj.empty_display_type = self.type
                obj.empty_display_size = 1 * self.scale
                #obj.displayColorGizmo = False
                obj.color = (self.color[0],self.color[1],self.color[2],1)
                obj.displayColorGizmo = self.displayColorGizmo
            
            if self.type == 'FITTED_CAGE':
                
                Xmin = -SPBoxXScale/2
                Ymin = -SPBoxYScale/2
                Zmin = -SPBoxZScale/2
                Xmax = SPBoxXScale/2
                Ymax = SPBoxYScale/2
                Zmax = SPBoxZScale/2
                
                if self.origin_at_cursor == True:
                    delta = Vector(SPBoxLocation) - Vector(context.scene.cursor.location)
                    Xmin+=delta[0]
                    Xmax+=delta[0]
                    Ymin+=delta[1]
                    Ymax+=delta[1]
                    Zmin+=delta[2]
                    Zmax+=delta[2]
                
                verts = [(Xmax, Ymin, Zmin),
                         (Xmax, Ymax, Zmin),
                         (Xmin, Ymin, Zmin),
                         (Xmin, Ymax, Zmin),
                         (Xmax, Ymin, Zmax),
                         (Xmax, Ymax, Zmax),
                         (Xmin, Ymin, Zmax),
                         (Xmin, Ymax, Zmax)]
                         
                edges = [(0,1),
                         (0,2),
                         (2,3),
                         (1,3),
                         (5,7),
                         (7,6),
                         (6,4),
                         (4,5),
                         (1,5),
                         (4,0),
                         (2,6),
                         (3,7)
                        ]
                        
                mesh = bpy.data.meshes.new(self.name)
                mesh.from_pydata(verts, edges, [])
                obj = bpy.data.objects.new('tmp_name', mesh)
                obj.color = (self.color[0],self.color[1],self.color[2],1)
                obj.displayColorGizmo = self.displayColorGizmo
                obj.isSuperParentCage = True
                
            favCollection.objects.link(obj)
            
            bpy.context.view_layer.objects.active = obj
            obj.select_set(True)
            
            
            
            
            SP = context.object
            if self.origin_at_cursor == False:
                SP.location = SPBoxLocation
            else:
                SP.location = context.scene.cursor.location
            
            SP.name = self.name
            SP.show_in_front = self.show_in_front
            bpy.context.evaluated_depsgraph_get().update()
            
            
            
            for o in missingObjects:
                o.parent = SP
                o.matrix_parent_inverse = SP.matrix_world.inverted()
                o.select_set(False)    
            
        
        
        return {'FINISHED'}



class SP_RefreshCage(Operator):
    bl_idname = "ap.refresh_super_parent_cage"
    bl_description = "Refresh Parent Cage" 
    bl_options = {"UNDO"}
    bl_label = "Refresh Parent Cage" 
    
    def execute(self, context):
        
        cage = context.object
        children = cage.children_recursive
        
        boundBoxXs = []
        boundBoxYs = []
        boundBoxZs = []
        
        for o in children:
            bb_vertices = [Vector(v) for v in o.bound_box]
            cage_mat = cage.matrix_world.inverted()
            mat = cage_mat @ o.matrix_world 
            world_bb_vertices = [mat @ v for v in bb_vertices]
            for i in world_bb_vertices:
                boundBoxXs.append(i[0])    
                boundBoxYs.append(i[1])    
                boundBoxZs.append(i[2])
            o.select_set(False)

        #delete verts
        bpy.ops.object.editmode_toggle()
        bpy.ops.mesh.select_all(action='SELECT')
        bpy.ops.mesh.delete(type='VERT')
      
        
        #add cube
        bpy.ops.mesh.primitive_cube_add()
        bpy.ops.mesh.delete(type='ONLY_FACE')
        bpy.ops.mesh.select_all(action='SELECT')
        bpy.ops.object.editmode_toggle()
            
        Xmin = min(boundBoxXs)
        Ymin = min(boundBoxYs)
        Zmin = min(boundBoxZs)
        Xmax = max(boundBoxXs)
        Ymax = max(boundBoxYs)
        Zmax = max(boundBoxZs)
        
        cage.data.vertices[1].co = (Xmax, Ymax, Zmin)
        cage.data.vertices[0].co = (Xmax, Ymin, Zmin)
        cage.data.vertices[2].co = (Xmin, Ymin, Zmin)
        cage.data.vertices[3].co = (Xmin, Ymax, Zmin)
        cage.data.vertices[5].co = (Xmax, Ymax, Zmax)
        cage.data.vertices[4].co = (Xmax, Ymin, Zmax)
        cage.data.vertices[6].co = (Xmin, Ymin, Zmax)
        cage.data.vertices[7].co = (Xmin, Ymax, Zmax)
       
        #bpy.context.view_layer.objects.active = cage
        #cage.select_set(True)
        return {'FINISHED'}

@persistent
def update_Mesh_Cages(self, context):
        if get_addon_prefs().automatic_cage:
            objects = bpy.context.scene.objects
            for obj in objects:
                if obj.isSuperParentCage:
                    cage = bpy.context.scene.objects[obj.name]
                    children = cage.children_recursive
                    
                    boundBoxXs = []
                    boundBoxYs = []
                    boundBoxZs = []
                    
                    for o in children:
                        bb_vertices = [Vector(v) for v in o.bound_box]
                        cage_mat = cage.matrix_world.inverted()
                        mat = cage_mat @ o.matrix_world 
                        world_bb_vertices = [mat @ v for v in bb_vertices]
                        for i in world_bb_vertices:
                            boundBoxXs.append(i[0])    
                            boundBoxYs.append(i[1])    
                            boundBoxZs.append(i[2])
                 
                    try:
                        Xmin = min(boundBoxXs)
                        Ymin = min(boundBoxYs)
                        Zmin = min(boundBoxZs)
                        Xmax = max(boundBoxXs)
                        Ymax = max(boundBoxYs)
                        Zmax = max(boundBoxZs)
                        
                        cage.data.vertices[1].co = (Xmax, Ymax, Zmin)
                        cage.data.vertices[0].co = (Xmax, Ymin, Zmin)
                        cage.data.vertices[2].co = (Xmin, Ymin, Zmin)
                        cage.data.vertices[3].co = (Xmin, Ymax, Zmin)
                        cage.data.vertices[5].co = (Xmax, Ymax, Zmax)
                        cage.data.vertices[4].co = (Xmax, Ymin, Zmax)
                        cage.data.vertices[6].co = (Xmin, Ymin, Zmax)
                        cage.data.vertices[7].co = (Xmin, Ymax, Zmax)
                    
                    except:
                        pass




# ------------------------------------------------------------------------
# ------------------------------------------------------------------------
#    CAGE GIZMOS
# ------------------------------------------------------------------------
# ------------------------------------------------------------------------

custom_shape_verts = ((0, 0, 0), (0, 0, 2))

class SP_Cage_Gizmo(Gizmo):
    bl_idname = "ap.cage_gizmo"
    __slots__ = (
        "custom_shape",
    )
    
    def setup(self):
        if not hasattr(self, "custom_shape"):
            self.custom_shape = self.new_custom_shape('TRIS', custom_shape_verts)
    
    def draw(self, context):
        self.draw_custom_shape(self.custom_shape)

    def draw_select(self, context, select_id):
        self.draw_custom_shape(self.custom_shape, select_id=select_id)


class SP_Cage_GizmoGroup(GizmoGroup):
    bl_idname = "ap.cage_gizmo_group"
    bl_label = "Cage Gizmo Widget"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'3D', 'PERSISTENT'}


    def setup(self, context):
        self.mesh_gizmos = []


    def refresh(self, context):
        
        objects=[ob for ob in bpy.context.view_layer.objects if (ob.visible_get() and ob.displayColorGizmo)]
        
        if len(self.gizmos.values()) != len(objects):
            for g in self.gizmos:
                self.gizmos.remove(g)
                self.mesh_gizmos = []
        
        i = 0
        
        for ob in objects:
            
            
            if i>len(self.mesh_gizmos)-1:
                mpr = self.gizmos.new(SP_Cage_Gizmo.bl_idname)
                mpr.hide_select = True
                mpr.use_draw_scale = False
                self.mesh_gizmos.append(mpr)
            
            mpr = self.mesh_gizmos[i]
            
            
            
            
            
            
            if ob.type == 'MESH':
                verts = ob.data.vertices
                if len(verts)>0:
                    edges = ob.data.edges
                    custom_shape_verts = []
                    
                    for e in edges:
                        custom_shape_verts.append(verts[e.vertices[0]].co)
                        custom_shape_verts.append(verts[e.vertices[1]].co)
                    mpr.custom_shape = mpr.new_custom_shape('LINES', custom_shape_verts)
                    mpr.matrix_basis = ob.matrix_world
                    
                    
            
            if ob.type == 'EMPTY':
                if ob.empty_display_type in {'PLAIN_AXES','IMAGE'}:
                    custom_shape_verts = ((0, 0, -1), (0, 0, 1),
                                          (0, -1, 0), (0, 1, 0),
                                          (-1, 0, 0), (1, 0, 0))
                
                if ob.empty_display_type == 'ARROWS':
                    custom_shape_verts = ((0, 0, 0), (0, 0, 1),
                                          (0, 0, 0), (0, 1, 0),
                                          (0, 0, 0), (1, 0, 0))
                
                if ob.empty_display_type == 'SINGLE_ARROW':
                    custom_shape_verts = ((0, 0, 0), (0, 0, 0.75),
                    (-0.035, -0.035, 0.75), (0.0, 0.0, 1.0), 
                    (-0.035, 0.035, 0.75), (0.0, 0.0, 1.0),
                    (0.035, -0.035, 0.75), (0.0, 0.0, 1.0),
                    (0.035, 0.035, 0.75),(0.0, 0.0, 1.0),
                    (-0.035, -0.035, 0.75),(-0.035, 0.035, 0.75),
                    (-0.035, -0.035, 0.75),(0.035, -0.035, 0.75),
                    (0.035, 0.035, 0.75),(-0.035, 0.035, 0.75),
                    (0.035, 0.035, 0.75),(0.035, -0.035, 0.75),
                                          )
                                          
                if ob.empty_display_type == 'CIRCLE':
                    custom_shape_verts =[]
                    edges = []
                    for i in range(63):
                        edges.append((i,i+1))
                    edges.append((63,0))
                    
                    vertices = []
                    
                    for count in range(0,64):
                        x = math.sin(count/64*math.radians(360)) * 1
                        z = math.cos(count/64*math.radians(360)) * 1
                        vertices.append((x, 0, z))
        
                                          
                    for e in edges:
                        custom_shape_verts.append(vertices[e[0]])
                        custom_shape_verts.append(vertices[e[1]])
                    custom_shape_verts = tuple(custom_shape_verts)
                
                
                
                if ob.empty_display_type == 'SPHERE':
                    custom_shape_verts =[]
                    edges = []
                    for i in range(0,63):
                        edges.append((i,i+1))
                        edges.append((i,i+1))
                    for i in range(64,127):
                        edges.append((i,i+1))
                        edges.append((i,i+1))
                    for i in range(128,191):
                        edges.append((i,i+1))
                        edges.append((i,i+1))
                    
                    edges.append((63,0))
                    edges.append((127,64))
                    edges.append((191,128))
                    
                    vertices = []
                    
                    for count in range(0,64):
                        x = math.sin(count/64*math.radians(360)) * 1
                        y = math.cos(count/64*math.radians(360)) * 1
                        vertices.append((x, y, 0))
                    
                    for count in range(0,64):
                        y = math.cos(count/64*math.radians(360)) * 1
                        z = math.sin(count/64*math.radians(360)) * 1
                        vertices.append((0, y, z))
                    
                    for count in range(0,64):
                        x = math.sin(count/64*math.radians(360)) * 1
                        z = math.cos(count/64*math.radians(360)) * 1
                        vertices.append((x, 0, z))
                    
                    for e in edges:
                        custom_shape_verts.append(vertices[e[0]])
                        custom_shape_verts.append(vertices[e[1]])
                    custom_shape_verts = tuple(custom_shape_verts)
                        
                        
                        
                        
                                                  
                if ob.empty_display_type == 'CUBE':
                    custom_shape_verts = ((-1.0, -1.0, -1.0),(-1.0, -1.0, 1.0), 
                                        (-1.0, 1.0, -1.0), (-1.0, 1.0, 1.0), 
                                        (1.0, -1.0, -1.0), (1.0, -1.0, 1.0),
                                        (1.0, 1.0, -1.0), (1.0, 1.0, 1.0),
                                                (-1.0, -1.0, -1.0), (-1.0, 1.0, -1.0),
                                                (1.0, 1.0, -1.0), (1.0, -1.0, -1.0),
                                                (-1.0, 1.0, -1.0), (1.0, 1.0, -1.0),
                                                (-1.0, -1.0, -1.0), (1.0, -1.0, -1.0),
                                                        (-1.0, -1.0, 1.0), (-1.0, 1.0, 1.0),
                                                        (1.0, 1.0, 1.0), (1.0, -1.0, 1.0),
                                                        (-1.0, 1.0, 1.0), (1.0, 1.0, 1.0),
                                                        (-1.0, -1.0, 1.0), (1.0, -1.0, 1.0),
                                          )
                if ob.empty_display_type == 'CONE':
                    
                    custom_shape_verts =[]
                    edges = [(0,8),(1,8),(2,8),(3,8),(4,8),(5,8),(6,8),(7,8),(0,1),(1,2),(2,3),(3,4),(4,5),(5,6),(6,7),(0,7)]
                    
                    vertices = [(2.220446049250313e-16, 0.0, -1.0), 
                    (0.7071067094802856, -5.960464477539063e-08, -0.7071067690849304), 
                    (0.9999999403953552, 2.220446049250313e-16, 2.220446049250313e-16), 
                    (0.7071067094802856, 5.960464477539063e-08, 0.7071067690849304), 
                    (-2.220446049250313e-16, 0.0, 1.0), 
                    (-0.7071067094802856, 5.960464477539063e-08, 0.7071067690849304), 
                    (-0.9999999403953552, -2.220446049250313e-16, -2.220446049250313e-16), 
                    (-0.7071067094802856, -5.960464477539063e-08, -0.7071067690849304), 
                    (-4.440892098500626e-16, 2.0, 0.0)
                                          ]
                    for e in edges:
                        custom_shape_verts.append(vertices[e[0]])
                        custom_shape_verts.append(vertices[e[1]])
                    custom_shape_verts = tuple(custom_shape_verts)
                                          
                mpr.custom_shape = mpr.new_custom_shape('LINES', custom_shape_verts)
                scale_matrix_x2 = ob.matrix_world.Scale(ob.empty_display_size, 4, (1.0, 0.0, 0.0))
                scale_matrix_y2 = ob.matrix_world.Scale(ob.empty_display_size, 4, (0.0, 1.0, 0.0))
                scale_matrix_z2 = ob.matrix_world.Scale(ob.empty_display_size, 4, (0.0, 0.0, 1.0))
                mpr.matrix_basis = ob.matrix_world @ scale_matrix_x2 @ scale_matrix_y2 @ scale_matrix_z2
               
            
            mpr.color = ob.color[0], ob.color[1],ob.color[2]
            if bpy.context.object == ob and ob.select_get() :
                mpr.alpha = 1 * ob.color[3]
                
            else: 
                mpr.alpha = 0.75 * ob.color[3]
            mpr.alpha*= bpy.context.space_data.overlay.show_overlays*ob.displayColorGizmo
            i+=1



class SP_PT_ColorGizmoPanel(bpy.types.Panel):
    bl_label = "Colored Wire Gizmo"
    bl_idname = "SP_PT_ColorGizmoPanel"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = 'object'
    #bl_parent_id = OBJECT_PT_display.bl_label
    bl_order = 12
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        
        
        obj = context.object
        obj_type = obj.type
        is_geometry = (obj_type in {'MESH', 'CURVE', 'SURFACE', 'META', 'FONT', 'VOLUME', 'CURVES', 'POINTCLOUD'})
        is_mesh = (obj_type in {'MESH', 'EMPTY'})
        has_bounds = (is_geometry or obj_type in {'LATTICE', 'ARMATURE'})
        is_wire = (obj_type in {'CAMERA', 'EMPTY'})
        is_empty_image = (obj_type == 'EMPTY' and obj.empty_display_type == 'IMAGE')
        is_dupli = (obj.instance_type != 'NONE')
        is_gpencil = (obj_type == 'GPENCIL')
        
        
        if is_mesh: #is_geometry or is_dupli or is_empty_image or is_gpencil:
            row = self.layout.row()
            row.prop(obj, "displayColorGizmo")
            row = self.layout.row()
            row.prop(obj, "color")
            
        else:
            row = self.layout.row()
            row.label(text = 'No Color Gizmo for this type of object yet', icon = "INFO")
            
        
        

# ------------------------------------------------------------------------
#    ICONS

preview_collections_SuperParent = {}


# ------------------------------------------------------------------------
# ------------------------------------------------------------------------
#    REGISTRATION
# ------------------------------------------------------------------------
# ------------------------------------------------------------------------

def register():
    
    def update(self, context):
        
        obj = bpy.context.object
        if obj !=None:
        
            if obj.type == 'MESH':
                bpy.context.object.data.update()
            
            else:
                obj.select_set(False) 
                obj.select_set(True)
    
    pcoll_SuperParent = bpy.utils.previews.new()
    
    icons_dir_SB = os.path.join(os.path.dirname(__file__), 'icons')
    pcoll_SuperParent.load('super_parent', os.path.join(icons_dir_SB, 'super_parent.png'), 'IMAGE')
    
    preview_collections_SuperParent['main'] = pcoll_SuperParent
    
    bpy.utils.register_class(SP_AddSuperParent)
    bpy.utils.register_class(SP_RefreshCage)
    bpy.utils.register_class(SP_Cage_Gizmo)
    bpy.utils.register_class(SP_Cage_GizmoGroup)
    bpy.utils.register_class(SP_PT_ColorGizmoPanel)
    bpy.utils.register_class(SuperParentPreferences)
    
    bpy.types.VIEW3D_MT_add.prepend(draw_superparent_menu)
    bpy.types.VIEW3D_MT_object.prepend(draw_cage_object)
    
    bpy.types.Object.isSuperParentCage = bpy.props.BoolProperty(default=False)
    bpy.types.Object.displayColorGizmo = bpy.props.BoolProperty(default=False, name= "Show Gizmo", update=update)
    
    bpy.app.handlers.depsgraph_update_post.append(update_Mesh_Cages)
    bpy.app.handlers.frame_change_post.append(update_Mesh_Cages)
    bpy.app.translations.register(__package__, GetTranslationDict())

def unregister():
    
    bpy.types.VIEW3D_MT_add.remove(draw_superparent_menu)
    bpy.types.VIEW3D_MT_object.remove(draw_cage_object)
    
    bpy.utils.unregister_class(SP_AddSuperParent)
    bpy.utils.unregister_class(SP_RefreshCage)
    bpy.utils.unregister_class(SP_Cage_Gizmo)
    bpy.utils.unregister_class(SP_Cage_GizmoGroup)
    bpy.utils.unregister_class(SP_PT_ColorGizmoPanel)
    bpy.utils.unregister_class(SuperParentPreferences)

    for pcoll in preview_collections_SuperParent.values():
        bpy.utils.previews.remove(pcoll)
    preview_collections_SuperParent.clear()
    
    del bpy.types.Object.isSuperParentCage
    del bpy.types.Object.displayColorGizmo
    
    bpy.app.handlers.depsgraph_update_post.remove(update_Mesh_Cages)
    bpy.app.handlers.frame_change_post.remove(update_Mesh_Cages)
    bpy.app.translations.unregister(__package__)

if __name__ == "__main__":
    register()